# -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*-
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is the Thunderbird Preferences System.
#
# The Initial Developer of the Original Code is
# Scott MacGregor.
# Portions created by the Initial Developer are Copyright (C) 2005
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Scott MacGregor <mscott@mozilla.org>
#   Karsten D¸sterloh <mnyromyr@tprac.de>
#   Postbox, Inc. <contact@postbox-inc.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

var gDisplayPane = {
  mInitialized: false,
  mTagListBox:  null,

  init: function ()
  {
    if ("arguments" in window && window.arguments[1] && document.getElementById(window.arguments[1]))
      document.getElementById("displayPrefs").selectedTab = document.getElementById(window.arguments[1]);
    else
    {
      var preference = document.getElementById("mail.preferences.display.selectedTabIndex");
      if (preference.value)
        document.getElementById("displayPrefs").selectedIndex = preference.value;
    }

    this.mInitialized = true;

    this.mTagListBox = document.getElementById('tagList');
    this.buildTagList();
    this.onTagSelectionChanged();
  },

  tabSelectionChanged: function ()
  {
    if (this.mInitialized)
      document.getElementById("mail.preferences.display.selectedTabIndex")
              .valueFromPreferences = document.getElementById("displayPrefs").selectedIndex;
  },

  fontOptionsDialog: function()
  {
    document.documentElement.openSubDialog("chrome://messenger/content/preferences/fonts.xul", "", null);
  },

  /**
   * update the delete, move up and move down buttons based on the tag selection
   */
  onTagSelectionChanged: function()
  {
    var entry = this.mTagListBox.selectedItem;
    document.getElementById('removeTagButton').disabled = !entry;
    document.getElementById('editTagButton').disabled = !entry;
    document.getElementById('moveTagUpButton').disabled = !entry || !this.mTagListBox.getPreviousItem(entry, 1);
    document.getElementById('moveTagDownButton').disabled = !entry || !this.mTagListBox.getNextItem(entry, 1);
  },

  // appends the tag to the tag list box
  appendTagItem: function(aTagName, aKey, aColor, aActive)
  {
    var entry = document.createElement('listitem');
    entry.setAttribute('value', aKey);
    entry.setAttribute('allowevents', 'true');  // activate the checkbox
    var tagCell = document.createElement('listcell');
    tagCell.setAttribute('label', aTagName);
    entry.appendChild(tagCell);

    var activeCell = document.createElement('listcell');
    activeCell.setAttribute('type', 'checkbox');
    activeCell.setAttribute('pack', 'center');
    activeCell.className = 'activeTagCheckbox';
    activeCell.setAttribute('checked', aActive);
    activeCell.addEventListener('click', _toggleActiveTagState, false);
    entry.appendChild(activeCell);

    entry.style.color = aColor;
    return this.mTagListBox.insertBefore(entry, null);
  },

  buildTagList: function()
  {
    var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
                               .getService(Components.interfaces.nsIMsgTagService);
    var tagArray = tagService.getAllTags({});
    for (var i = 0; i < tagArray.length; ++i)
    {
      var taginfo = tagArray[i];
      var item = this.appendTagItem(taginfo.tag, taginfo.key, taginfo.color, taginfo.active);
      // select the first item by default
      if (i == 0)
        this.mTagListBox.selectedItem = item;
    }
  },

  removeTag: function()
  {
    var index = this.mTagListBox.selectedIndex;
    if (index >= 0)
    {
      var itemToRemove = this.mTagListBox.getItemAtIndex(index);
      var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"].getService(Components.interfaces.nsIMsgTagService);
      tagService.deleteKey(itemToRemove.getAttribute("value"));
      this.mTagListBox.removeItemAt(index);
      var numItemsInListBox = this.mTagListBox.getRowCount();
      this.mTagListBox.selectedIndex = index < numItemsInListBox ? index : numItemsInListBox - 1;
    }
  },

  /**
   * Open the edit tag dialog
   */
  editTag: function()
  {
    var index = this.mTagListBox.selectedIndex;
    if (index >= 0)
    {
      var tagElToEdit = this.mTagListBox.getItemAtIndex(index);
      var args = {result: "", keyToEdit: tagElToEdit.getAttribute("value"), okCallback: editTagCallback};
      var dialog = window.openDialog(
                  "chrome://messenger/content/newTagDialog.xul",
                  "",
                  "chrome,titlebar,modal",
                  args);
    }
  },

  addTag: function()
  {
    var args = {result: "", okCallback: addTagCallback};
    var dialog = window.openDialog(
                 "chrome://messenger/content/newTagDialog.xul",
                 "",
                 "chrome,titlebar,modal",
                 args);
  },

  /**
   * Move the selected tag one position up or down in the tagList's child order.
   * This reordering may require changing ordinal strings, which will happen
   * when we write tag data to the preferences system in the OKHandler.
   */
  moveTag: function (aMoveUp)
  {
    // Move the selected tag one position up or down in the tagList's child order.
    // This reordering may require changing ordinal strings, which will happen
    // when we write tag data to the preferences system in the OKHandler.
    var entry = this.mTagListBox.selectedItem;
    // UpdateTagInfo(entry.taginfo, entry); // remember changed values
    var successor = aMoveUp ? this.mTagListBox.getPreviousItem(entry, 1)
                            : this.mTagListBox.getNextItem(entry, 2);
    entry.parentNode.insertBefore(entry, successor);
    this.mTagListBox.selectedItem = entry;

    // update the ordinal values based on the new keys
    var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"].
                     getService(Components.interfaces.nsIMsgTagService);

    // reset the ordinal value on the tags
    // we really should be doing this in some sort of on ok handler for the preferences panel
    for (var i = 0; i < this.mTagListBox.getRowCount(); i++)
      tagService.setOrdinalForKey(this.mTagListBox.getItemAtIndex(i).getAttribute("value"), i);
  },
};

function addTagCallback(aName, aColor)
{
  var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
                    .getService(Components.interfaces.nsIMsgTagService);
  tagService.addTag(aName, aColor, '', true /* active */);

  var item = gDisplayPane.appendTagItem(aName, tagService.getKeyForTag(aName), aColor, true);
  var tagListBox = document.getElementById('tagList');
  tagListBox.ensureElementIsVisible(item);
  tagListBox.selectItem(item);
  tagListBox.focus();
}

function editTagCallback()
{
  // update the values of the selected item
  var tagListEl = document.getElementById('tagList');
  var index = tagListEl.selectedIndex;
  if (index >= 0)
  {
    var tagElToEdit = tagListEl.getItemAtIndex(index);
    var key = tagElToEdit.getAttribute("value");
    var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
                     .getService(Components.interfaces.nsIMsgTagService);
    // update the color and label elements
    tagElToEdit.childNodes[0].setAttribute("label", tagService.getTagForKey(key));
    tagElToEdit.style.color = tagService.getColorForKey(key);
  }
}

/**
 * @param aEvent the listcell the event originated on
 */
function _toggleActiveTagState(aEvent)
{
  if (aEvent.button == 0 && !aEvent.ctrlKey && !aEvent.shiftKey && !aEvent.altKey && !aEvent.metaKey)
  {
    var tagService = Components.classes["@mozilla.org/messenger/tagservice;1"]
                             .getService(Components.interfaces.nsIMsgTagService);
    var toggleActive = aEvent.target.getAttribute('checked') != 'true';
    tagService.setActiveForKey(aEvent.target.parentNode.getAttribute('value'), toggleActive);
    return aEvent.target.setAttribute('checked', aEvent.target.getAttribute('checked') != 'true');
  }
}
